<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icon id.
 *
 * @property-read int $width The raw value.
 * @property-read int $height The raw value.
 */
final class Size {
	/**
	 * The width value of the icon.
	 *
	 * @var int
	 */
	private readonly int $width;
	/**
	 * The height value of the icon.
	 *
	 * @var int
	 */
	private readonly int $height;

	/**
	 * The constructor.
	 *
	 * @param mixed $width The width of the icon.
	 * @param mixed $height The height of the icon.
	 * @throws \InvalidArgumentException If the width or height is not numeric or less than 1.
	 */
	public function __construct( mixed $width, mixed $height ) {
		if ( ! is_int( $width ) && ! is_string( $width ) ) {
			throw new \InvalidArgumentException( 'The width must be an integer or a string.' );
		}
		if ( ! is_numeric( $width ) ) {
			throw new \InvalidArgumentException( 'The width must be numeric.' );
		}

		$width = intval( $width );
		if ( $width < 1 ) {
			throw new \InvalidArgumentException( 'The width must be greater than 0.' );
		}

		if ( ! is_int( $height ) && ! is_string( $height ) ) {
			throw new \InvalidArgumentException( 'The height must be an integer or a string.' );
		}
		if ( ! is_numeric( $height ) ) {
			throw new \InvalidArgumentException( 'The height must be numeric.' );
		}

		$height = intval( $height );
		if ( $height < 1 ) {
			throw new \InvalidArgumentException( 'The height must be greater than 0.' );
		}

		$this->width  = $width;
		$this->height = $height;
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'width' => $this->width,
			'height' => $this->height,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * This function converts the object to an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			'width'  => $this->width,
			'height' => $this->height,
		);
	}
}
